from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from sliced import datasets
from sliced import base
from sliced import externals

from sliced.sir import *
from sliced.save import *
from sliced.version import *
